@@ -3,9 +3,15 @@
 # Utilitary functions to use inside the module
 
 import numpy as np
+import xarray as xr
+
+
+def nbp2km(nbp):
+    return np.sqrt(510e6 / (nbp * 1e3))
+
 
 ### ======================================================= ###
-###          Utilitary data manipulation functions          ###
+###          Utilitary _data manipulation functions          ###
 ### ======================================================= ###
 
 
@@ -33,7 +39,7 @@ def sign_change_detect(A):
     Parameters
     ----------
     A : list or 1D array
-        The data in which we want to detect the sign change
+        The _data in which we want to detect the sign change
 
     Returns
     -------
@@ -45,5 +51,18 @@ def sign_change_detect(A):
     return np.min(np.where(change == True))
 
 
+def hist2d(bdd, weights=None, n=None, resolution=4):
+    if n == None:
+        n = (bdd.time.dt.year.max() - bdd.time.dt.year.min() + 1) * 12
+    H, X, Y = np.histogram2d(
+        bdd.lon,
+        bdd.lat,
+        weights=weights,
+        bins=[int(360 / resolution), int(180 / resolution)],
+        range=((0, 360), (-90, 90)),
+    )
+    return (X[1:] + X[:-1]) / 2, (Y[1:] + Y[:-1]) / 2, H / n
+
+
 if __name__ == "__main__":
     pass
